# Live Exam OS Builder

This project builds a customized ISO image used during live exam with nexus. The following features are supported: root filesystem encryption, PXE booting, secure boot, OS and filesystem configuration.

## Usage

A Makefile is provided to build the nexus live exam ISO image. This Makefile takes care of running the build script, `build.sh`, inside a container in order to avoid any potential corruption of the host system. Indeed, the script binds directories such as `/dev`, `/sys`, etc. to the created filesystem. If a problem occurs during this process, the host system might be affected and possibly corrupted.

Consequently, make absolutely sure to NOT DIRECTLY RUN `build.sh` on the host system! Instead, use `make` as described below:

1. First, prepare the container image required by the build process:
   ```bash
   make prepare
   ```

2. Then, run the ISO image build process with the desired config:
   ```bash
   make build conf=<env>
   ```
   Where `<env>` is the path to a file containing the config to use when building the image. The config is specified through environment variables described in the next section.

## Configuration

Here is a sample configuration file listing all supported variables. In this example, encryption is disabled. To activate it, uncomment the block under `[Encryption settings]`.

```bash
# [General settings]
# Final ISO image to produce
export ISO_NAME="nexus.iso"
# Whether to produce extra messages
export VERBOSE=true
# Output directory for generated files
export OUTPUT="output"
# Directory where to download packages for the initial root filesystem
export CACHE_FS="cache"

# [nexus-exam settings]
# nexus server ip and port
export SERVER="127.0.0.1:1077"
# Path, within the current directory, to the public certificate file used by nexus-exam
export CERT="ca-cert.pem"
# Default user for nexus-exam
export EXAM_USER='luke@force.net'
# Default password for nexus-exam
export EXAM_PWD='r2d2'

# [Encryption settings]
# Enable LUKS encryption with passphrase
# export LUKS_PASSPHRASE='passphrase'
# List of additional keys
# export ADDITIONAL_KEYS='key1 key2'
# Copy the rootfs to RAM (instead of being stored on the usb key)
#export RAM="squashfs_tmpfs"

# [PXE settings]
# Set the url of the PXE server and generate an initramfs, kernel and squashfs for PXE to use
# Use the url of the directory where squashfs is stored, not the direct link to it
#export PXE_URL="127.0.0.1:8000"
```

## Makefile Targets

The default target is `help`, which shows the available targets:

```
$ make
Available targets:
prepare     Prepare the container image used by the build script
build       Build the ISO image by running the build script inside a container
            require the conf variable which specifies the config file to use, e.g.
            make build conf=my_awsome_env
bios        Boot the ISO as a harddisk in legacy mode (BIOS)
bios_cd     Boot the ISO on a CD-ROM in legacy mode (BIOS)
uefi        Boot the ISO as a harddisk in UEFI mode
```

Note that a QEMU virtual machine is used when booting the ISO image.

## Project Components

### Scripts

 - **build.sh**: Main script to build the ISO image and configure the environment.
 - **tools/functions.sh**: Contains helper functions used by the build.sh script to streamline operations and ensure consistency.

### Configuration Files

The configuration files are organized in the `config/` directory and include the following subdirectories:

- **config/00-bootloader**: Contains bootloader configuration files.
  - grub.cfg, grub.efi.cfg, limine.conf

- **config/01-packages_install**: Lists packages to be installed.
  - packages

- **config/02-customisation**: Contains customization files for the filesystem.
  - etc/, home/, usr/

- **config/03-post_install**: Contains post-installation scripts to further customize the environment.
  - 1000-create-user.sh, 2000-enable_services.sh, 3000-nexus_exam.sh, 5000-push_challenge.sh, 8000-initramfs.sh, 1000-disable_root.sh, 2500-disable_services.sh, 4000-firewall.sh, 5000-pxe.sh, 9999-cleanup.sh

### Wifi Connection

Each wifi connection must reside in a `.nmconnection` file located in `config/02-customisation/etc/NetworkManager/system-connections/`.

Here are two example files, `wifi1.nmconnection` and `wifi2.nmconnection`. Make sure that whatever configuration files you copy in that directory **cannot be read by anyone**, but root:
```
chmod 600 config/02-customisation/etc/NetworkManager/system-connections/*.nmconnection
```

Content of `wifi1.nmconnection`:
```
[connection]
# Name of the connection as shown by Network Manager
id=example1
# Random UUID referencing the connection
uuid=8a065b14-beb3-4f28-97b5-f560c368ae12
type=wifi

[wifi]
# Whether the SSID is hidden or not (true/false)
hidden=false
# Name of the WIFI SSID
ssid=the_empire_strickes_back

[wifi-security]
key-mgmt=wpa-eap

[802-1x]
eap=peap;
identity=luke_skywalker
password=you_are_not_ready_young_padawan
phase2-auth=mschapv2

[ipv4]
method=auto

[ipv6]
addr-gen-mode=stable-privacy
method=auto
```

Content of `wifi2.nmconnection`:
```
[connection]
# Name of the connection as shown by Network Manager
id=example2
# Random UUID referencing the connection
uuid=8a065b14-beb3-4f28-97b5-f560c368ae12
type=wifi

[wifi]
# Whether the SSID is hidden or not (true/false)
hidden=true
# Name of the WIFI SSID
ssid=return_of_the_jedi
mode=infrastructure

[wifi-security]
auth-alg=open
key-mgmt=wpa-psk
psk=may_the_force_be_with_you

[ipv4]
method=auto

[ipv6]
addr-gen-mode=stable-privacy
method=auto
```
## Notes

- Ensure that all required dependencies are installed before running the script or Makefile.
- For UEFI testing, verify that the OVMF firmware is installed on your system (e.g., `/usr/share/OVMF/x64/OVMF.fd`).
- When specifying a cache directory or a directory for downloading the filesystem, avoid using `/tmp` in the container, as it is often set with the noexec attribute, which can prevent tools like debootstrap from working properly. Choose a directory with execution permissions instead.
